import 'package:codigojaguar/restFunction.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class BotonRest extends StatefulWidget {
  /// Boton de código Jaguar con RestFunction para realizar lladamas REST a una [url]
  /// con un [metodo] y si la respuesta es true se realiza una [action].
  /// [estilo] define el estilo del boton, el [contendio] del boton, puede estar ligado a un
  /// form con un [formkey] y enseña un mensaje de error [errorStyle] cuando la respuesta es false.
  /// También se puede mostrar un [showModalLoader] con un [stringCargando].
  const BotonRest(
      {Key key,
      @required this.contenido,
      this.estilo,
      this.url,
      this.method,
      // this.action,
      this.formkey,
      this.arrayData,
      this.errorStyle,
      this.showModalLoader = true,
      this.stringCargando = 'Cargando...',
      this.action,
      this.token,
      this.showSuccess = false,
      this.succesStyle,
      this.habilitado = true,
      this.restriccion = false,
      this.restriccionStr,
      this.restriccionStyle,
      this.primerAction})
      : super(key: key);

  final Function(dynamic) action;
  final Function primerAction;
  final Widget contenido;
  final BoxDecoration estilo;
  final String url;
  final String method;
  // final Function action;
  final formkey;
  final dynamic arrayData;
  final TextStyle errorStyle;
  final TextStyle succesStyle;
  final bool showModalLoader;
  final String stringCargando;
  final String token;
  final bool showSuccess;
  final bool habilitado;
  final bool restriccion;
  final String restriccionStr;
  final TextStyle restriccionStyle;

  @override
  _BotonRestState createState() => _BotonRestState();
}

class _BotonRestState extends State<BotonRest> {
  bool load = false;
  bool error = false;
  bool restiction = false;
  String errorString = 'Algo salio mal';

  bool success = false;
  String succesString = 'Todo bien';

  Rest restFunction = Rest();

  String response;

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        widget.showSuccess
            ? success
                ? Padding(
                    padding: const EdgeInsets.only(bottom: 15),
                    child: Text(
                      succesString,
                      textAlign: TextAlign.center,
                      style: widget.succesStyle,
                    ),
                  )
                : Container()
            : Container(),
        error
            ? Padding(
                padding: const EdgeInsets.only(bottom: 15),
                child: Text(
                  errorString,
                  textAlign: TextAlign.center,
                  style: widget.errorStyle,
                ),
              )
            : Container(),
        widget.restriccion
            ? restiction
                ? Padding(
                    padding: const EdgeInsets.only(bottom: 15),
                    child: Text(
                      widget.restriccionStr,
                      textAlign: TextAlign.center,
                      style: widget.restriccionStyle,
                    ),
                  )
                : Container()
            : Container(),
        Container(
          decoration: widget.estilo,
          child: Material(
            clipBehavior: Clip.hardEdge,
            color: Colors.transparent,
            child: InkWell(
              highlightColor: Colors.grey.withOpacity(0.2),
              onTap: () {
                if (!load) {
                  if (widget.habilitado) {
                    if (widget.primerAction != null) {
                      widget.primerAction();
                    }
                    if (widget.formkey != null) {
                      if (widget.formkey.currentState.validate()) {
                        widget.formkey.currentState.save();
                        setState(() {
                          load = true;
                        });
                        if (widget.showModalLoader) {
                          loadingDialog(context, widget.stringCargando);
                        }
                        restFunction
                            .restService(widget.arrayData, widget.url,
                                widget.token, widget.method)
                            .then((serverResp) {
                          setState(() {
                            response = serverResp.toString();
                          });
                          if (serverResp['status'] == 'server_true') {
                            Navigator.pop(context);
                            widget.action(serverResp);
                            setState(() {
                              load = false;
                              error = false;
                              restiction = false;
                              success = true;
                              succesString = serverResp['message'].toString();
                            });
                          } else {
                            Navigator.pop(context);
                            setState(() {
                              load = false;
                              error = true;
                              restiction = false;
                              success = false;
                              errorString = serverResp['message'].toString();
                            });
                          }
                        });
                      }
                    } else {
                      setState(() {
                        load = true;
                      });
                      if (widget.showModalLoader) {
                        loadingDialog(context, widget.stringCargando);
                      }
                      restFunction
                          .restService(widget.arrayData, widget.url,
                              widget.token, widget.method)
                          .then((serverResp) {
                        setState(() {
                          response = serverResp.toString();
                        });
                        if (serverResp['status'] == 'server_true') {
                          Navigator.pop(context);
                          widget.action(serverResp);
                          setState(() {
                            load = false;
                            error = false;
                            restiction = false;
                            success = true;
                            succesString = serverResp['message'].toString();
                          });
                        } else {
                          Navigator.pop(context);
                          setState(() {
                            load = false;
                            error = true;
                            restiction = false;
                            success = false;
                            errorString = serverResp['message'].toString();
                          });
                        }
                      });
                    }
                  } else {
                    setState(() {
                      restiction = true;
                    });
                  }
                }
              },
              child: Container(
                  padding: EdgeInsets.symmetric(vertical: 10, horizontal: 5),
                  alignment: Alignment.center,
                  child: load
                      ? Container(
                          height: 18,
                          width: 18,
                          child: CircularProgressIndicator(
                            backgroundColor: Colors.white,
                            strokeWidth: 3,
                          ))
                      : widget.contenido),
            ),
          ),
        )
      ],
    );
  }

  Future<String> function() async {
    return response;
  }

  loadingDialog(BuildContext context, String stringCarga) {
    AlertDialog alert = AlertDialog(
      content: WillPopScope(
        onWillPop: () {},
        child: Row(
          children: [
            CircularProgressIndicator(),
            Container(
                margin: EdgeInsets.only(left: 7),
                child: Text(stringCarga.toString())),
          ],
        ),
      ),
    );
    showDialog(
      barrierDismissible: false,
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }
}

class EntradaTexto extends StatefulWidget {
  final String tipo;
  final FormFieldSetter<String> onSaved;
  final FormFieldSetter<String> onChanged;
  final Function onEditingComplete;
  final bool habilitado;
  final int lineasMax;
  final int longMaxima;
  final int longMinima;
  final String valorInicial;
  final TextCapitalization textCapitalization;
  final InputDecoration estilo;
  final TextInputAction action;
  final TextInputType tipoEntrada;
  final GlobalKey<FormState> formkey;
  final String validacionStr;
  final bool requerido;

  EntradaTexto(
      {Key key,
      this.tipo = 'textoCorto',
      this.onSaved,
      this.onChanged,
      this.habilitado = true,
      this.lineasMax = 1,
      this.longMaxima,
      this.longMinima,
      this.valorInicial,
      this.textCapitalization = TextCapitalization.sentences,
      this.estilo,
      this.action = TextInputAction.next,
      this.tipoEntrada = TextInputType.name,
      this.onEditingComplete,
      this.formkey,
      this.validacionStr = 'Campo olbigatorio',
      this.requerido = true})
      : super(key: key);

  @override
  _EntradaTextoState createState() => _EntradaTextoState();
}

class _EntradaTextoState extends State<EntradaTexto> {
  Function(String) typeValidator;
  int maxCharacters;

  @override
  Widget build(BuildContext context) {
    switch (widget.tipo) {
      case "texto":
        typeValidator = (value) {
          if (widget.longMinima != null) {
            if (value.isEmpty || value.length < widget.longMinima) {
              return widget.validacionStr;
            } else {
              return null;
            }
          } else {
            if (value.isEmpty || value.length < 10) {
              return widget.validacionStr;
            } else {
              return null;
            }
          }
        };
        setState(() {
          maxCharacters = widget.longMaxima == null ? 20 : widget.longMaxima;
        });
        break;
      default:
        typeValidator = (value) {
          if (widget.longMinima != null) {
            if (value.isEmpty || value.length < widget.longMinima) {
              return widget.validacionStr;
            } else {
              return null;
            }
          } else {
            if (value.isEmpty || value.length < 10) {
              return widget.validacionStr;
            } else {
              return null;
            }
          }
        };
        setState(() {
          maxCharacters = widget.longMaxima == null ? 20 : widget.longMaxima;
        });
    }

    return TextFormField(
      decoration: widget.estilo,
      validator: widget.requerido ? typeValidator : null,
      maxLength: maxCharacters,
      onSaved: widget.onSaved,
      obscureText: widget.tipo == 'password' ? true : false,
      enabled: widget.habilitado,
      maxLines: widget.lineasMax,
      initialValue: widget.valorInicial,
      textCapitalization: widget.textCapitalization,
      textInputAction: widget.action,
      keyboardType: widget.tipoEntrada,
      inputFormatters: widget.tipo == 'numeroINT' ||
              widget.tipo == 'moneda' ||
              widget.tipo == 'telefono'
          ? [
              // DecimalTextInputFormatter(),
              FilteringTextInputFormatter.allow(RegExp("[0-9 .]"))
            ]
          : null,
    );
  }
}

class BotonSimple extends StatefulWidget {
  const BotonSimple({
    @required this.contenido,
    this.action,
    this.estilo,
  });

  final Widget contenido;
  final Function action;
  final BoxDecoration estilo;

  @override
  _BotonSimpleState createState() => _BotonSimpleState();
}

class _BotonSimpleState extends State<BotonSimple> {
  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: widget.estilo,
      child: Material(
        color: Colors.transparent,
        child: InkWell(
          highlightColor: Colors.grey.withOpacity(0.2),
          onTap: widget.action,
          child: Container(
            padding: EdgeInsets.symmetric(vertical: 10, horizontal: 5),
            alignment: Alignment.center,
            // height: 50.0,
            child: widget.contenido,
          ),
        ),
      ),
    );
  }
}
